/*
 *
 * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
 * @license AGPL-3.0
 *
 * This code is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with OX App Suite. If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
 *
 * Any use of the work other than as authorized under this license or copyright law is prohibited.
 *
 */

import { generateSimpleViteManifest, injectApp, mockConfig, mockFetch, mockRedis } from './util.js'
import { expect } from 'chai'
import * as td from 'testdouble'
import RedisMock from 'ioredis-mock'

describe('Responses contain custom headers', function () {
  let app

  before(async function () {
    await mockConfig({ baseUrls: ['http://ui-server/'] })
    mockFetch({
      'http://ui-server': {
        '/manifest.json': generateSimpleViteManifest({
          'example.js': {},
          'main.css': {},
          'index.html': {
            file: 'index.html.js',
            isEntry: true,
            imports: ['example.js'],
            css: ['main.css']
          }
        }),
        '/example.js': () => new Response('this is example', { headers: { 'content-type': 'application/javascript' } }),
        '/index.html.js': () => new Response('this is index.html.js', { headers: { 'content-type': 'application/javascript' } }),
        '/index.html': () => new Response('<html><head></head><body>it\'s me</body></html>', { headers: { 'content-type': 'text/html' } }),
        '/main.css': () => new Response('.foo { color: #000; }', { headers: { 'content-type': 'text/css' } })
      }
    });
    (await mockRedis()).isReady()
    app = await injectApp()
  })

  afterEach(async function () {
    await new RedisMock().flushdb()
  })

  after(async function () {
    await app.close()
    td.reset()
  })

  it('index.html has version', async function () {
    const response = await app.inject({ url: '/index.html' })
    expect(response.statusCode).to.equal(200)
    expect(response.headers.version).to.equal('3215668592')
  })

  it('serves requested version', async function () {
    const response = await app.inject({ url: '/index.html.js', headers: { version: '123456' } })
    expect(response.statusCode).to.equal(200)
    expect(response.headers.version).to.equal('123456')
    expect(response.headers['latest-version']).to.equal('3215668592')
  })

  it('javascript file contains dependencies', async function () {
    const response = await app.inject({ url: '/index.html.js' })
    expect(response.statusCode).to.equal(200)
    expect(response.headers.dependencies[0]).to.equal('main.css')
  })

  describe('with different files', function () {
    beforeEach(async function () {
      td.reset()
      await new RedisMock().flushdb()

      await mockConfig({ baseUrls: ['http://ui-server/'] })
      await mockRedis()
      mockFetch({
        'http://ui-server': {
          '/manifest.json': generateSimpleViteManifest({
            'index.html': {}
          }),
          '/index.html': () => new Response('<html><head></head><body>it\'s me</body></html>', { headers: { 'content-type': 'text/html' } })
        }
      })
      app = await injectApp()
    })

    it('index.html has version', async function () {
      const response = await app.inject({ url: '/index.html' })
      expect(response.statusCode).to.equal(200)
      // important here is, that it is different than in the test without meta.json
      expect(response.headers.version).to.equal('3961519424')
    })
  })

  describe('with meta.json', function () {
    beforeEach(async function () {
      td.reset()
      await new RedisMock().flushdb()

      await mockConfig({ baseUrls: ['http://ui-server/'] })
      await mockRedis()
      mockFetch({
        'http://ui-server': {
          '/manifest.json': generateSimpleViteManifest({
            'index.html': {}
          }),
          '/index.html': () => new Response('<html><head></head><body>it\'s me</body></html>', { headers: { 'content-type': 'text/html' } }),
          '/meta.json': { commitSha: '1234567890' }
        }
      })
      app = await injectApp()
    })

    it('index.html has version', async function () {
      const response = await app.inject({ url: '/index.html' })
      expect(response.statusCode).to.equal(200)
      // important here is, that it is different than in the test without meta.json
      expect(response.headers.version).to.equal('1487554813')
    })
  })

  describe('with different meta.json', function () {
    beforeEach(async function () {
      td.reset()
      await new RedisMock().flushdb()

      await mockConfig({ baseUrls: ['http://ui-server/'] })
      await mockRedis()
      mockFetch({
        'http://ui-server': {
          '/manifest.json': generateSimpleViteManifest({
            'index.html': {}
          }),
          '/index.html': () => new Response('<html><head></head><body>it\'s me</body></html>', { headers: { 'content-type': 'text/html' } }),
          '/meta.json': { commitSha: '0987654321' }
        }
      })
      app = await injectApp()
    })

    it('index.html has version', async function () {
      const response = await app.inject({ url: '/index.html' })
      expect(response.statusCode).to.equal(200)
      // important here is, that it is different than in the test without meta.json
      expect(response.headers.version).to.equal('319344871')
    })
  })
})
